<?php
/**
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
class Montonio_Gateway_Model_Method_Blik extends Montonio_Gateway_Model_Method_Payments
{

    const METHOD_CODE = 'montonio_blik';
    const GATEWAY_METHOD_CODE = 'blik';
    const GATEWAY_LOCALES = [
        'en_US' => 'en_US',
        'et_EE' => 'et',
        'lt_LT' => 'lt',
        'lv_LV' => 'lv',
        'ru_RU' => 'ru',
        'de_DE' => 'de',
        'en_GB' => 'en_US',
        'fi_FI' => 'fi',
        'pl_PL' => 'pl',
    ];
    const GATEWAY_PREF_COUNTRIES = [
        'PL' => 'PL'
    ];

    protected $_formBlockType               = 'montonio/blik_form';
    protected $_infoBlockType               = 'montonio/payments_info';
    protected $_code                        = self::METHOD_CODE;
    protected $_allowCurrencyCode           = ['PLN'];
    protected $_canManageRecurringProfiles  = false;
    protected $_canUseInternal              = false;

    /**
     *
     * @param  $order
     * @param  $payment
     * @return array
     */
    public function prepPayload($order)
    {

        $storeId        = $order->getStoreId();
        $returnUrl      = $this->getConfigData('return_url', $storeId);
        $hookUrl        = $this->getConfigData('return_hook_url', $storeId);
        $storeLocale    = Mage::app()->getLocale()->getLocaleCode();
        //montonio only supports following locales

        $locales = self::GATEWAY_LOCALES;
        $localeCode = (isset($locales[$storeLocale])) ? $locales[$storeLocale] : $locales['en_US'];

        $data = [
            'merchantReference' => (string)$order->getIncrementId(),
            'returnUrl'         => (string)Mage::getUrl($returnUrl),
            'notificationUrl'   => (string)Mage::getUrl($hookUrl),
            'currency'          => (string)$order->getOrderCurrency()->toString(),
            'grandTotal'        => (float)sprintf("%.2f", $order->getGrandTotal()),
            'locale'            => $localeCode,
        ];

        $data['billingAddress'] = $this->getAddressPayload($order->getBillingAddress(), $order->getCustomerEmail());

        if($order->getQuote()->getIsVirtual()) {
            $shippingAddress = $order->getBillingAddress();
        } else {
            $shippingAddress = $order->getShippingAddress();
        }
        $data['shippingAddress'] = $this->getAddressPayload($shippingAddress, $order->getCustomerEmail());

        foreach ($order->getAllItems() as $item) {
            if(!$item->isDeleted() && !$item->getParentItem()) {
                $data['lineItems'][] = [
                    'name'       => (string)Mage::helper('core')->escapeHtml($item->getName()),
                    'quantity'   => (float)$item->getQtyOrdered(),
                    'finalPrice' => (float)($item->getPriceInclTax() - $item->getDiscountAmount()),
                ];
            }
        }

        $data['payment'] = [
            'method'        => self::GATEWAY_METHOD_CODE,
            'methodDisplay' => (string)Mage::helper('core')->escapeHtml($this->getConfigData('title')),
            'amount'        => $data['grandTotal'], // Yes, this is the same as order->grandTotal.
            'currency'      => (string)$data['currency'], // This must match the currency of the order.
            'methodOptions' => [
                //'paymentReference'   => '1010109',
                'paymentDescription' => (string)Mage::helper('montonio')
                    ->__('Payment for order "%s"', $data['merchantReference']),
                'preferredLocale'    => $localeCode
            ]
        ];

        return $data;
    }

    /**
     *
     * @inheritdoc
     */
    public function canUseForCountry($countryCode) {

        $result = false;

        if($countryCode == 'PL') {
            $result =  true;
        }

        return $result;
    }

    /**
     * Check method for processing with base currency
     *
     * @param string $currencyCode
     * @return boolean
     */
    public function canUseForCurrency($currencyCode)
    {
        $result = false;

        if($currencyCode == 'PLN') {
            $result =  true;
        }

        return $result;
    }

}
