<?php
/**
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

require_once(Mage::getBaseDir('lib') . DS . 'Montonio' . DS . 'Split/MontonioSplitSDK.php');

class Montonio_Gateway_Model_Method_Split extends Mage_Payment_Model_Method_Abstract
{
    const METHOD_CODE           = 'montonio_split';
    const METHOD_ALL_COUNTRIES  = 'ALL';

    const SPLIT_METHOD_COMBINED = 'Combined';

    protected $_code                       = self::METHOD_CODE;
    protected $_formBlockType               = 'montonio/split_form';
    protected $_allowCurrencyCode          = ['EUR'];
    protected $_canManageRecurringProfiles = false;
    protected $_canUseInternal             = false;

    /**
     * @inheritdoc
     */
    public function getOrderPlaceRedirectUrl()
    {
        $url         = Mage::getUrl('checkout/cart');
        $environment = $this->getConfigData('environment');

        try {
            $info   = $this->getInfoInstance();
            $data   = $info->getAdditionalInformation();

            if (!empty($data[$this->getCode().'_data'])) {
                $sdk = new MontonioSplitSDK(
                    $this->getConfigData('access_key'),
                    $this->getConfigData('secret_key'),
                    $environment
                );

                $sdk->setPaymentData($data[$this->getCode().'_data']);
                $url = $sdk->getPaymentUrl();
            } else {
                throw new Mage_Core_Exception('no payment data for building gateway redirect url');
            }
        } catch (Exception $e) {
            Mage::logException($e);
        }
        return $url;
    }

    /**
     *
     * @param  $order
     * @param  $payment
     * @return array
     */
    public function prepPayload($order)
    {
        $data      = [];
        $storeId   = $order->getStoreId();
        $time      = Mage::getModel('core/date')->timestamp(time());

        /* system data */
        $data['exp']        = $time + (60 * 10);
        $data['access_key'] = $this->getConfigData('access_key', $storeId);

        $returnUrl = $this->getConfigData('return_url', $storeId);
        $hookUrl   = $this->getConfigData('return_hook_url', $storeId);

        $data['merchant_return_url'] = Mage::getUrl($returnUrl);
        $data['merchant_notification_url'] = Mage::getUrl($hookUrl);

        $storeLocale = Mage::app()->getLocale()->getLocaleCode();

        //montonio only supports following locales
        $locales = [
            'en_US' => 'en_US',
            'et_EE' => 'et',
            'lt_LT' => 'lt',
            'ru_RU' => 'ru'
        ];

        $localeCode = (isset($locales[$storeLocale])) ? $locales[$storeLocale] : $storeLocale;

        $data['preselected_locale'] = $localeCode;

        /*order data*/
        $data['amount'] = (float)sprintf("%.2f", $order->getGrandTotal());
        $data['currency'] = $order->getOrderCurrency()->toString();
        $data['merchant_reference'] = $order->getIncrementId();

        if ($order->getCustomerEmail()) {
            $data['checkout_email'] = $order->getCustomerEmail();
        }

        if ($order->getBillingAddress()->getFirstname()) {
            $data['checkout_first_name'] = $order->getBillingAddress()->getFirstname();
        }

        if ($order->getBillingAddress()->getLastname()) {
            $data['checkout_last_name'] = $order->getBillingAddress()->getLastname();
        }

        if ($order->getBillingAddress()->getTelephone()) {
            $data['checkout_phone_number'] =  $order->getBillingAddress()->getTelephone();
        }

        if ($order->getBillingAddress()->getCity()) {
            $data['checkout_city'] = $order->getBillingAddress()->getCity();
        }

        if (is_array($order->getBillingAddress()->getStreet()) && $order->getBillingAddress()->getStreet()[0]) {
            $data['checkout_address'] = $order->getBillingAddress()->getStreet()[0];
        }

        if ($order->getBillingAddress()->getPostcode()) {
            $data['checkout_postal_code'] = $order->getBillingAddress()->getPostcode();
        }
        $data['merchant_name'] = $this->getConfigData('merchant_name');
        $data['checkout_products'] = array();

        $data['preselected_loan_period'] = null;

        // add products to $draftData
        foreach ($order->getAllItems() as $key => $product) {
            $data['checkout_products'][] = array(
                'product_name'  =>         $product->getName(),
                'product_price' => (float) $product->getPriceInclTax(),
                'quantity'      => (int)   $product->getQtyOrdered()
            );
        }

        // add shipping cost to $draftData
        $shippingCost = $order->getShippingAmount();
        if ($shippingCost > 0) {
            $data['checkout_products'][] = array(
                'quantity'      => 1,
                'product_name'  => 'SHIPPING',
                'product_price' => (float) $shippingCost
            );
        }

        return $data;
    }

    /**
     *
     * @param string $token
     * @param string $storeId
     * @return array
     */
    public function validateIncomingData($token, $storeId = null)
    {
        $accessKey = $this->getConfigData('access_key', $storeId);
        $data      = (array)MontonioSplitSDK::decodePaymentToken($token, $this->getConfigData('secret_key'));
        try {
            $this->validateSignedData($data, $accessKey);
        } catch (\Exception $e) {
            Mage::logException($e);
            $data = [];
        }

        return $data;
    }

    /**
     *
     * @param array $data
     * @param string $accesKey
     * @return boolean
     */
    public function validateSignedData($data, $accesKey)
    {

        $requiredKeys = explode(',',$this->getConfigData('validate_fields'));
        $time         = Mage::getModel('core/date')->timestamp(time());
        $data         = (array)$data;

        foreach($requiredKeys as $k => $v) {
            if(!isset($data[$v])) {
                Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on missing field in message: "%s"', $v));
            }
        }

        if ($data['access_key'] !== $accesKey) {
            Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on "%s" validation', 'access_key'));

        }
        if ($data['exp'] < $time) {
            Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on "%s" validation', 'exp'));
        }

        return true;
    }

    public function handleOrderStatus($data, $response)
    {
        $order  = Mage::getModel('sales/order')
            ->loadByIncrementId($data['merchant_reference']);
        $helper = Mage::helper('montonio');

        if (!isset($order['increment_id']) ||
            $order['increment_id'] !== $data['merchant_reference'] ||
            $order['status'] !== 'pending'
        ) {
            $response['status'] = $order['status'];
            return $response;
        }

        $quote = Mage::getModel('sales/quote')
            ->load($order['quote_id']);

        if ($data['status'] === 'finalized') {
            $newStatus = Mage_Sales_Model_Order::STATE_PROCESSING;
            $message = $helper->__('Payment accepted in payment gateway for "%s"', $data['amount']);
            $quote->setIsActive(0);
        } else {
            $newStatus = Mage_Sales_Model_Order::STATE_CANCELED;
            $message = $helper->__('Payment cancelled in payment gateway');
            $quote->setIsActive(1);
        }

        $transaction = Mage::getModel('core/resource_transaction');
        $transaction->addObject($quote);

        $order->setState($newStatus, true, $message, false);
        $order->setCanSendNewEmailFlag(true);

        $canCreateInvoice = $this->getConfigData('create_invoice', $order->getStoreId());

        //add invoice to order
        if (
            $order->getState() === Mage_Sales_Model_Order::STATE_PROCESSING &&
            $order->canInvoice() &&
            $canCreateInvoice
        ) {

            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_OFFLINE);
            $invoice->register();
            $invoice->addComment($message,false,false);
            $invoice->getOrder()->setIsInProcess(true);

            $transaction->addObject($invoice);

            $order = $invoice->getOrder();

        }


        $transaction->addObject($order);
        $transaction->save();

        try {

            $canOrderEmail = $this->getConfigData('order_email_after', $order->getStoreId());

            if($order->getCanSendNewEmailFlag() && $canOrderEmail) {
                //for 1.9.* it turned to queue based method
                if(method_exists ($order, 'queueNewOrderEmail')) {
                    $order->queueNewOrderEmail();
                } else {
                    //for anything lower than 1.9 we need to use the old method
                    if(method_exists ($order, 'sendNewOrderEmail')) {
                        $order->sendNewOrderEmail();
                    }
                }
            }

        } catch (Exception $e) {
            Mage::logException($e);
        }

        try {

            $canInvoiceEmail = $this->getConfigData('invoice_email_after', $order->getStoreId());

            if($canCreateInvoice && is_object($invoice) && $canInvoiceEmail) {
                if(method_exists ($invoice, 'sendEmail')) {
                    $invoice->sendEmail();
                }
            }

        } catch (Exception $e) {
            Mage::logException($e);
        }

        $response = [
            'success' => true,
            'error' => false,
            'status' => $newStatus,
            'message' => $message
        ];

        if ($newStatus === Mage_Sales_Model_Order::STATE_PROCESSING) {
            $response['data'] = [
                'order_increment_id' => $order['increment_id'],
                'order_id' => $order['entity_id'],
                'quote_id' => $order['quote_id'],
            ];
        }

        return $response;
    }

    /**
     *
     * @inheritdoc
     */
    public function canUseForCountry($countryCode) {

        $result = false;

        if(
            $this->getConfigData('country') !== '' &&
            (
                $countryCode == $this->getConfigData('country') ||
                $this->getConfigData('country') === self::METHOD_ALL_COUNTRIES ||
                $this->getConfigData('allowall') == 1
            )
        ) {
            $result =  true;
        } else if ($this->getConfigData('country') == '') {
            $result =  parent::canUseForCountry($countryCode);
        }

        return $result;
    }

    /**
     * Check method for processing with base currency
     *
     * @param string $currencyCode
     * @return boolean
     */
    public function canUseForCurrency($currencyCode)
    {
        $result = false;

        if($currencyCode == 'EUR') {
            $result =  true;
        }

        return $result;
    }

}